#include "customview.h"



CustomView::~CustomView()
{
    doublyOccupiedOrbitals = 0;
    if(selectedSpin != NULL)
        delete selectedSpin;
    if(selectedOrbital != NULL)
        delete selectedOrbital;
    for(int i = scene()->items().size() - 1; i >= 0 ; i--)
    {
        QGraphicsItem *item = scene()->items().at(i);
        delete item;
        //scene()->removeItem(item);
    }
    if(parent != 0)
        delete parent;
}

CustomView::CustomView(QObject *parent) :
    QGraphicsView((QGraphicsScene*)parent)
{
    this->parent = parent;
    this->selectedSpin = NULL;
    this->selectedOrbital = NULL;
}

void CustomView::closeEvent(QCloseEvent *e)
{
    emit signalViewClosed();
}

void CustomView::startImageAnimation(PictureItem *item)
{
    int dx = 100;
    qreal x0 = item->pos().x();
    qreal y0 = item->pos().y();
    QPointF p1, p2;

    p1 = QPointF(x0,y0);
    p2 = QPointF(10,10);
    QPropertyAnimation *animation = item->animatePosition(p1, p2, abs(dx));
    animation->start();
}

//set FREE the status of the orbital where the spin was before
void CustomView::updateOrbitalStatus(int index, Side side, SpinType spin)
{
    for(int i = 0; i < scene()->items().size() ; i++)
    {
        QGraphicsItem *item = scene()->items().at(i);
        string className = typeid(*item).name();
        if(className.compare("11PictureItem") == 0)
        {
            PictureItem *pictItem = (PictureItem *)item;
            if(pictItem->getType() == ORBITAL)
            {
                Orbital orbital = pictItem->orbital;
                if(orbital.getSide() == side && orbital.getIndex() == index)
                {
                    orbital.setStatus(spin, FREE);
                    pictItem->orbital = orbital;
                    break;
                }
            }
        }
     }
}

void CustomView::mousePressEvent(QMouseEvent *event)
{
    QGraphicsView::mousePressEvent(event);
    QGraphicsItem *item = itemAt(event->pos());

    if (!item) {
        //cout << "You didn't click on an item.";
        return;
    }
    string className = typeid(*item).name();

    if(className.compare("11PictureItem") != 0)
    {
        //cout << "It is not an object\n";
        return;
    }
    else
    {
        PictureItem *pictItem = (PictureItem *)item;
        if(pictItem->getType() == SPIN)
        {
            selectedSpin = pictItem;
        }
        else
        {
            selectedOrbital = pictItem;
            if(selectedSpin != NULL && selectedSpin->spin.isCompatibleWithOrbital(selectedOrbital->orbital))
            {
                QPointF endPoint;
                endPoint.setY(selectedOrbital->orbital.spinY());
                endPoint.setX(selectedSpin->pos().x());
                QPropertyAnimation *anim = selectedSpin->animatePosition(selectedSpin->pos(), endPoint, 100);
                anim->start();
                selectedOrbital->orbital.setStatus(selectedSpin->spin.getSpin(), OCCUPIED);
                updateOrbitalStatus(selectedSpin->spin.getFinalOrbital(),selectedSpin->spin.getSide(),selectedSpin->spin.getSpin());
                selectedSpin->spin.setFinalOrbital(selectedOrbital->orbital.getIndex());
                selectedSpin = NULL;
                selectedOrbital = NULL;
                collectData();
            }
            else if(selectedSpin == NULL)
            {
                //no spin selected
                selectedOrbital = NULL;
            }
            else if(!selectedSpin->spin.isCompatibleWithOrbital(selectedOrbital->orbital))
            {
                //incompatible spin-orbital
                selectedOrbital = NULL;
            }
        }
    }
}

//draw an up spin on a given side and orbital
void CustomView::putUpSpin(Side side, int pos)
{
    int x0;
    if(side == LEFT)
        x0 = LEFT_ORBITAL_X0;
    else
        x0 = RIGHT_ORBITAL_X0;

    int y1 = ORBITAL_Y0 + (NB_ORBITALS - pos) * ORBITAL_DELTA;
    stringstream str;

    str << SC_PATH << "up_arrow.png";
    QPixmap pm(str.str().c_str());
    PictureItem *item = new PictureItem(QSize(ARROW_WIDTH, ARROW_HEIGHT), pm.scaled(ARROW_WIDTH, ARROW_HEIGHT, Qt::KeepAspectRatio));
    item->setPos(x0 + ARROW_DIST - ARROW_WIDTH / 2, y1 - ARROW_HEIGHT / 2);
    item->setType(SPIN);
    Spin spin;
    spin.setSpin(UP);
    spin.setSide(side);
    spin.setInitialOrbital(pos);
    spin.setFinalOrbital(pos);
    item->spin = spin;
    scene()->addItem(item);
}

//draw a down spin on a given side and orbital
void CustomView::putDownSpin(Side side, int pos)
{
    int x0;
    if(side == LEFT)
        x0 = LEFT_ORBITAL_X0;
    else
        x0 = RIGHT_ORBITAL_X0;
    int y1 = ORBITAL_Y0 + (NB_ORBITALS - pos) * ORBITAL_DELTA;
    stringstream str;
    str << SC_PATH << "down_arrow.png";
    QPixmap pm1(str.str().c_str());
    PictureItem *item1 = new PictureItem(QSize(ARROW_WIDTH, ARROW_HEIGHT), pm1.scaled(ARROW_WIDTH, ARROW_HEIGHT, Qt::KeepAspectRatio));
    item1->setPos(x0 + ORBITAL_LEN - ARROW_DIST - ARROW_WIDTH / 2, y1 - ARROW_HEIGHT / 2);
    item1->setType(SPIN);
    Spin spin;
    spin.setSpin(DOWN);
    spin.setSide(side);
    spin.setInitialOrbital(pos);
    spin.setFinalOrbital(pos);
    item1->spin = spin;
    scene()->addItem(item1);
}

//draw an orbital
void CustomView::putOrbital(Orbital orbital, int x0, int y1)
{
    stringstream str;
    str << SC_PATH << "orbital.png";
    QPixmap pm(str.str().c_str());
    PictureItem *item = new PictureItem(QSize(ORBITAL_LEN, ORBITAL_HEIGHT), pm.scaled(ORBITAL_LEN, ORBITAL_HEIGHT, Qt::KeepAspectRatio));
    item->setPos(x0, y1 - 10);
    item->setType(ORBITAL);
    item->orbital = orbital;
    scene()->addItem(item);
}

//update number of occupied orbitals and redraw them
void CustomView::initSpins(int nbr)
{
    doublyOccupiedOrbitals = nbr;
    clearAndReplace();
}

//clear old orbitals and fill them with the new number
void CustomView::clearAndReplace()
{

    for(int i = scene()->items().size() - 1; i >= 0 ; i--)
    {
        QGraphicsItem *item = scene()->items().at(i);
        string className = typeid(*item).name();
        if(className.compare("11PictureItem") == 0)
        {
            PictureItem *pictItem = (PictureItem *)item;
            if(pictItem->getType() == SPIN)
            {
                scene()->removeItem(scene()->items().at(i));
            }
            else//mark all the orbitals free when resetting the configuration
            {
                pictItem->orbital.setUpStatus(FREE);
                pictItem->orbital.setDownStatus(FREE);
            }
        }
    }

    //add the spins missing
    for(int i = 1; i <= doublyOccupiedOrbitals; i++)
    {
        putUpSpin(LEFT, i);
        putUpSpin(RIGHT, i);
        putDownSpin(LEFT, i);
        putDownSpin(RIGHT, i);
    }

    //set status of the filled orbitals as OCCUPIED
    for(int i = scene()->items().size() - 1; i >= 0 ; i--)
    {
        QGraphicsItem *item = scene()->items().at(i);
        string className = typeid(*item).name();
        if(className.compare("11PictureItem") == 0)
        {
            PictureItem *pictItem = (PictureItem *)item;
            if(pictItem->getType() == ORBITAL)
            {
                if(pictItem->orbital.getIndex() <= doublyOccupiedOrbitals)
                {
                    pictItem->orbital.setUpStatus(OCCUPIED);
                    pictItem->orbital.setDownStatus(OCCUPIED);
                }
            }
        }
    }
}

void CustomView::collectData()
{
    Configuration leftConfig;
    Configuration rightConfig;
    leftConfig.nbDoublyOccupied = doublyOccupiedOrbitals;
    rightConfig.nbDoublyOccupied = doublyOccupiedOrbitals;

    string res = "This is the result";

    int leftHolesIndex = 0, rightHolesIndex = 0;

    for(int i = scene()->items().size() - 1; i >= 0 ; i--)
    {
        QGraphicsItem *item = scene()->items().at(i);
        string className = typeid(*item).name();
        if(className.compare("11PictureItem") == 0)
        {
            PictureItem *pictItem = (PictureItem *)item;

            //put in the configuration the orbitals with holes
            if(pictItem->getType() == ORBITAL)
            {
                Orbital orbital = pictItem->orbital;
                int idx = orbital.getIndex();

                if(orbital.getIndex() <= doublyOccupiedOrbitals)
                {
                    if(orbital.getUpStatus() == FREE)
                    {
                        if(orbital.getSide() == LEFT)
                        {
                            leftConfig.holes[leftHolesIndex++] = idx;
                        }
                        else
                        {;
                            rightConfig.holes[rightHolesIndex++] = idx;
                        }
                    }
                    if(orbital.getDownStatus() == FREE)
                    {
                        if(orbital.getSide() == LEFT)
                        {
                            leftConfig.holes[leftHolesIndex++] = -idx;
                        }
                        else
                        {
                            rightConfig.holes[rightHolesIndex++] = -idx;
                        }
                    }
                }
                continue;
            }
            //put in the configuration the spins that were moved
            Spin spin = pictItem->spin;
            if(spin.getFinalOrbital() > doublyOccupiedOrbitals)
            {
                if(spin.getSide() == LEFT)
                {
                    leftConfig.excitationDegree++;
                    if(spin.getSpin() == UP)
                        leftConfig.parts[leftConfig.excitationDegree - 1] = spin.getFinalOrbital();
                    else
                        leftConfig.parts[leftConfig.excitationDegree - 1] = -spin.getFinalOrbital();
                }
                else
                {
                    rightConfig.excitationDegree++;
                    if(spin.getSpin() == UP)
                        rightConfig.parts[rightConfig.excitationDegree - 1] = spin.getFinalOrbital();
                    else
                        rightConfig.parts[rightConfig.excitationDegree - 1] = -spin.getFinalOrbital();
                }
            }
        }
    }
    cout << "LEFT config: "<< leftConfig.toString() << endl;
    cout << "RIGHT config: " << rightConfig.toString() << endl;
    cout << "------------------\n";
    char result[1024];
    int resultLen = 3;
    xmatel_routine_(leftConfig.nbDoublyOccupied, leftConfig.excitationDegree, leftConfig.holes, leftConfig.parts,
                   rightConfig.excitationDegree, rightConfig.holes, rightConfig.parts, result, resultLen);
    string res1 = "";
    if(doublyOccupiedOrbitals > 0)
        res1 = putEndCharacter(result, resultLen);
    emit setResult(res1.c_str());

}
