#include "mainpage.h"

MainPage::MainPage(QWidget *parent) :
    QWidget(parent)
{
    mainLayout = new QGridLayout();
    mainSplitter.setOrientation(Qt::Vertical);
    init();
    mainLayout->addWidget(&mainSplitter);
    this->setLayout(mainLayout);
}

MainPage::~MainPage()
{
    delete splitterLayout;
    delete mainLayout;
}


void MainPage::drawOrbitals(QGraphicsScene &scene, Side side)
{

    int x0;
    if(side == LEFT)
        x0 = LEFT_ORBITAL_X0;
    else
        x0 = RIGHT_ORBITAL_X0;

    for(int j = NB_ORBITALS; j > 0; j--)
    {
        int i = j - 1;
        int y1 = ORBITAL_Y0 + i * ORBITAL_DELTA;

        stringstream str;
        str << NB_ORBITALS - i;

        QGraphicsTextItem * text = new QGraphicsTextItem();
        text->setPlainText(str.str().c_str());
        text->setDefaultTextColor(Qt::gray);
        text->setPos(NUMBERING_X, y1 - 10);
        scene.addItem(text);

        Orbital orbital;
        orbital.setIndex(NB_ORBITALS - i);
        orbital.setSide(side);
        orbital.setX0(x0);
        orbital.setY0(y1);
        if(side == LEFT)
            spinsGraphicsView.leftOrbitals.push_back(orbital);
        else
            spinsGraphicsView.rightOrbitals.push_back(orbital);
        spinsGraphicsView.putOrbital(orbital, x0, y1);
    }
}


void MainPage::addImageOnScene(QGraphicsScene &scene, string path, int x, int y, int width, int height)
{
    QPixmap qpm1 = QPixmap(path.c_str());
    qpm1 = qpm1.scaled(width, height, Qt::KeepAspectRatio);
    QGraphicsPixmapItem *pm1 = new QGraphicsPixmapItem(qpm1);
    pm1->setPos(x, y);
    scene.addItem(pm1);
}

void MainPage::init()
{
    splitterLayout = new QGridLayout();
    mainSplitter.setLayout(splitterLayout);

    labelDoublyOccupiedOrbitals.setText("Doubly occupied orbitals:");
    labelDoublyOccupiedOrbitals.setGeometry(10, -20, 180, 30);

    spinBoxDoublyOccupiedOrbitals.setRange(0, 10);
    spinBoxDoublyOccupiedOrbitals.setGeometry(10, 30, 50, 30);

    spinsGraphicsScene.addWidget(&labelDoublyOccupiedOrbitals);
    spinsGraphicsScene.addWidget(&spinBoxDoublyOccupiedOrbitals);
    spinsGraphicsScene.setSceneRect(0, 0, WIDTH - DELTA, HEIGHT * RATIO - DELTA);

    helpButton.setText("Help");
    helpButton.setGeometry(spinsGraphicsScene.width() - 100, 0, 80, 30);
    connect(&helpButton, SIGNAL(clicked()), this, SIGNAL(helpButtonPressed()));
    spinsGraphicsScene.addWidget(&helpButton);

    theoryButton.setText("Theory");
    theoryButton.setGeometry(spinsGraphicsScene.width() - 100, 60, 80, 30);
    spinsGraphicsScene.addWidget(&theoryButton);
    connect(&theoryButton, SIGNAL(clicked()), this, SIGNAL(theoryButtonPressed()));

    reInitButton.setText("Init");
    reInitButton.setGeometry(spinsGraphicsScene.width() - 100, spinsGraphicsScene.height()-50, 80, 30);
    spinsGraphicsScene.addWidget(&reInitButton);

    spinsGraphicsView.setScene(&spinsGraphicsScene);
    spinsGraphicsView.setGeometry(0, 0, WIDTH, HEIGHT * RATIO);
    connect(&spinBoxDoublyOccupiedOrbitals, SIGNAL(valueChanged(int)), &spinsGraphicsView, SLOT(initSpins(int)));
    connect(&reInitButton, SIGNAL(clicked()), &spinsGraphicsView, SLOT(clearAndReplace()));
    //connect(&reInitButton, SIGNAL(clicked()), this, SLOT(clearFormula()));
    connect(&reInitButton, SIGNAL(clicked()), &spinsGraphicsView, SLOT(collectData()));

    formulaTextEdit.setReadOnly(true);
    connect(&spinsGraphicsView,SIGNAL(setResult(QString)), &formulaTextEdit, SLOT(setPlainText(QString)));
    connect(&spinBoxDoublyOccupiedOrbitals, SIGNAL(valueChanged(int)),&spinsGraphicsView, SLOT(collectData()));
    //adding the 3 signs
    stringstream str;
    str << SC_PATH << "lower_than.png";
    cout << str.str() << endl;
    addImageOnScene(spinsGraphicsScene, string(/*"./images/lower_than.png"*/str.str().c_str()), 10, IMAGES_Y0);
    str.str("");
    str << SC_PATH << "H.png";
    addImageOnScene(spinsGraphicsScene, string(/*"./images/H.png"*/str.str().c_str()), LEFT_ORBITAL_X0 + ORBITAL_LEN + 5, IMAGES_Y0);
    str.str("");
    str << SC_PATH << "greater_than.png";
    addImageOnScene(spinsGraphicsScene, string(/*"./images/greater_than.png"*/str.str().c_str()), RIGHT_ORBITAL_X0 + ORBITAL_LEN, IMAGES_Y0);

    splitterLayout->addWidget(&spinsGraphicsView);
    splitterLayout->addWidget(&formulaTextEdit);

    drawOrbitals(spinsGraphicsScene, LEFT);
    drawOrbitals(spinsGraphicsScene, RIGHT);

    QList<int> sizes;
    sizes.append(HEIGHT * 0.8);
    sizes.append(HEIGHT * (1.0 - 0.8));
    mainSplitter.setSizes(sizes);
}

void MainPage::clearFormula()
{
    formulaTextEdit.setPlainText("");
}
